import * as fs from 'fs';
import * as path from 'path';
import { TextDocument } from 'vscode';

export interface ContentItem {
    id: string,
    text: string
}

export class Util {

    public static async getContentItemsFromDocument(document: TextDocument): Promise<ContentItem[]> {
        let filePath: string = path.resolve(document.uri.fsPath.replace(/\/[^\/]+?$/g, "").replace(/\\[^\\]+?$/g, ""));
        let manifestPath: string = path.join(filePath, "MANIFEST.json");
        let manifestFound: boolean = false;

        while (filePath.length > 0 && !manifestFound) {
            if (fs.existsSync(manifestPath)) {
                manifestFound = true;
                break;
            }
            
            filePath = path.resolve(filePath.replace(/\/[^\/]+?[\/]?$/g, ""));
            manifestPath = path.join(filePath, "MANIFEST.json");
        }

        let contentItems: ContentItem[] = [];

        try {
            if (manifestFound) {
                contentItems = await Util.getContentItems(JSON.parse(fs.readFileSync(manifestPath, { encoding: 'utf8' })), filePath);
            } else {
                contentItems.push({
                    id: document.uri.toString(),
                    text: document.getText()
                });
            }
        } catch (err) {
            console.log("issue while parsing for content items: " + err);
        }
        
        return contentItems;

    }
    
    public static async getContentItems(manifest: any, srcDirectoryPath: string): Promise<ContentItem[]> {
        return new Promise<ContentItem[]> ((resolve, reject) => {
            try {
                let stories: string[] = manifest.include;
                let pathToRegex: {[key: string]: string} = {};
    
                for (let storyRegex of stories) {
                    let extractFileRegex: RegExp = /([\S]+\/)?([^\/]+?.abc)$/g;
                    let matchedPath: any = extractFileRegex.exec(storyRegex);
    
                    if (matchedPath != null) {
                        let postDir: string = matchedPath[1]? `/${matchedPath[1]}`: "";
                        if (!pathToRegex[`${srcDirectoryPath}${postDir}`]) {
                            pathToRegex[`${srcDirectoryPath}${postDir}`] = "";
                        } else {
                            pathToRegex[`${srcDirectoryPath}${postDir}`] += "|"
                        }
    
                        pathToRegex[`${srcDirectoryPath}${postDir}`] += `(?:\\b${matchedPath[2].replace(/\./g, "\\.").replace(/\*/g, ".*")}\\b)`;
                    }
                    
                }
    
                let combinedStoryContent: ContentItem[] = [];
    
                for (let searchDirectory of Object.keys(pathToRegex)) {
                    let files: string[] = fs.readdirSync(searchDirectory);
                    if (!files) {
                        throw `[Import ERROR] Cannot find story content files: ${JSON.stringify(stories, null, 4)}`;
                    }
                
                    for (var i = 0; i < files.length; i++) {
                        let file: string = files[i];
                        if (file.match(pathToRegex[searchDirectory])) {
                            combinedStoryContent.push({
                                id: path.join(searchDirectory, file),
                                text: fs.readFileSync(path.join(searchDirectory, file), { encoding: 'utf8' })
                            });
                        }
                    }
                }
            
                resolve(combinedStoryContent);
            } catch (err) {
                reject(err);
            }
        });
    }
}